<?php /*

 ocPortal
 Copyright (c) ocProducts, 2004-2012

 See text/EN/licence.txt for full licencing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license		http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright	ocProducts Ltd
 * @package		redirects_editor
 */

/**
 * Module page class.
 */
class Module_admin_redirects
{

	/**
	 * Standard modular info function.
	 *
	 * @return ?array	Map of module info (NULL: module is disabled).
	 */
	function info()
	{
		$info=array();
		$info['author']='Chris Graham';
		$info['organisation']='ocProducts';
		$info['hacked_by']=NULL;
		$info['hack_version']=NULL;
		$info['version']=4;
		$info['locked']=true;
		$info['update_require_upgrade']=1;
		return $info;
	}

	/**
	 * Standard modular uninstall function.
	 */
	function uninstall()
	{
		$GLOBALS['SITE_DB']->drop_if_exists('redirects');
	}

	/**
	 * Standard modular install function.
	 *
	 * @param  ?integer	What version we're upgrading from (NULL: new install)
	 * @param  ?integer	What hack version we're upgrading from (NULL: new-install/not-upgrading-from-a-hacked-version)
	 */
	function install($upgrade_from=NULL,$upgrade_from_hack=NULL)
	{
		if (is_null($upgrade_from))
		{
			$GLOBALS['SITE_DB']->create_table('redirects',array(
				'r_from_page'=>'*ID_TEXT',
				'r_from_zone'=>'*ID_TEXT',
				'r_to_page'=>'ID_TEXT',
				'r_to_zone'=>'ID_TEXT',
				'r_is_transparent'=>'BINARY',
			));

			$GLOBALS['SITE_DB']->query_insert('redirects',array('r_from_page'=>'rules','r_from_zone'=>'site','r_to_page'=>'rules','r_to_zone'=>'','r_is_transparent'=>1));
			$GLOBALS['SITE_DB']->query_insert('redirects',array('r_from_page'=>'rules','r_from_zone'=>'forum','r_to_page'=>'rules','r_to_zone'=>'','r_is_transparent'=>1));
			$GLOBALS['SITE_DB']->query_insert('redirects',array('r_from_page'=>'authors','r_from_zone'=>'collaboration','r_to_page'=>'authors','r_to_zone'=>'site','r_is_transparent'=>1));
		}

		if ((is_null($upgrade_from)) || ($upgrade_from<3))
		{
			$zones=find_all_zones();
			if (!in_array('site',$zones)) $zones[]='site';
			foreach ($zones as $zone)
			{
				if (!file_exists(get_file_base().'/'.$zone.'/pages/comcode/'.fallback_lang().'/panel_top.txt'))
					$GLOBALS['SITE_DB']->query_insert('redirects',array('r_from_page'=>'panel_top','r_from_zone'=>$zone,'r_to_page'=>'panel_top','r_to_zone'=>'','r_is_transparent'=>1));
			}
		}

		if ((is_null($upgrade_from)) || ($upgrade_from<4))
		{
			$zones=find_all_zones();
			if (!in_array('site',$zones)) $zones[]='site';
			foreach ($zones as $zone)
			{
				if (!file_exists(get_file_base().'/'.$zone.'/pages/comcode/'.fallback_lang().'/panel_bottom.txt'))
					$GLOBALS['SITE_DB']->query_insert('redirects',array('r_from_page'=>'panel_bottom','r_from_zone'=>$zone,'r_to_page'=>'panel_bottom','r_to_zone'=>'','r_is_transparent'=>1),false,true);
			}
		}
	}

	/**
	 * Standard modular entry-point finder function.
	 *
	 * @return ?array	A map of entry points (type-code=>language-code) (NULL: disabled).
	 */
	function get_entry_points()
	{
		return array('misc'=>'REDIRECTS');
	}

	/**
	 * Standard modular run function.
	 *
	 * @return tempcode	The result of execution.
	 */
	function run()
	{
		require_lang('redirects');

		$type=get_param('type','misc');

		if ($type=='misc') return $this->gui();
		if ($type=='actual') return $this->actual();

		return new ocp_tempcode();
	}

	/**
	 * The UI for managing redirects.
	 *
	 * @return tempcode		The UI
	 */
	function gui()
	{
		require_css('redirects_editor');

		$title=get_screen_title('REDIRECTS');
		$post_url=build_url(array('page'=>'_SELF','type'=>'actual'),'_SELF');
		$fields=new ocp_tempcode();
		$rows=$GLOBALS['SITE_DB']->query_select('redirects',array('*'));
		$num_zones=$GLOBALS['SITE_DB']->query_value('zones','COUNT(*)');
		require_code('zones3');
		foreach ($rows as $i=>$row)
		{
			$from_zones=($num_zones>50)?new ocp_tempcode():nice_get_zones($row['r_from_zone']);
			$to_zones=($num_zones>50)?new ocp_tempcode():nice_get_zones($row['r_to_zone']);
			$fields->attach(do_template('REDIRECTE_TABLE_REDIRECT',array('_GUID'=>'fd1ea392a98e588bb1f553464d315ef0','I'=>strval($i),'FROM_ZONE'=>$row['r_from_zone'],'TO_ZONE'=>$row['r_to_zone'],'TO_ZONES'=>$to_zones,'FROM_ZONES'=>$from_zones,'FROM_PAGE'=>$row['r_from_page'],'TO_PAGE'=>$row['r_to_page'],'TICKED'=>$row['r_is_transparent']==1,'NAME'=>'is_transparent_'.strval($i))));
		}
		$default=explode(':',get_param('page_link',':'),2);
		$zones=($num_zones>50)?new ocp_tempcode():nice_get_zones($default[0]);
		$new=do_template('REDIRECTE_TABLE_REDIRECT',array('_GUID'=>'cbf0eb4f745a6bf7b10e1f7d6d95d10f','I'=>'new','FROM_ZONE'=>'','TO_ZONE'=>'','TO_ZONES'=>$zones,'FROM_ZONES'=>$zones,'FROM_PAGE'=>$default[1],'TO_PAGE'=>'','TICKED'=>false,'NAME'=>'is_transparent_new'));

		require_code('form_templates');
		list($warning_details,$ping_url)=handle_conflict_resolution();

		$notes=get_long_value('notes');
		if (is_null($notes)) $notes='';

		return do_template('REDIRECTE_TABLE_SCREEN',array('_GUID'=>'2a9add73f6dd0b8288c0c84fc7242763','NOTES'=>$notes,'PING_URL'=>$ping_url,'WARNING_DETAILS'=>$warning_details,'TITLE'=>$title,'FIELDS'=>$fields,'NEW'=>$new,'URL'=>$post_url));
	}

	/**
	 * The actualiser for managing redirects.
	 *
	 * @return tempcode		The UI
	 */
	function actual()
	{
		$title=get_screen_title('REDIRECTS');

		$found=array();
		foreach ($_POST as $key=>$val)
		{
			if (!is_string($val)) continue;

			if (get_magic_quotes_gpc()) $val=stripslashes($val);

			if ((substr($key,0,10)=='from_page_') && ($val!=''))
			{
				$their_i=array_search($val,$found);
				$i=substr($key,10);
				if (($their_i!==false) && (post_param('from_zone_'.$i)==post_param('from_zone_'.strval($their_i))))
					warn_exit(do_lang_tempcode('DUPLICATE_PAGE_REDIRECT',post_param('from_zone_'.$i).':'.$val));
				$found[$i]=$val;
			}
		}

		$GLOBALS['SITE_DB']->query_delete('redirects');
		persistent_cache_empty();

		foreach ($found as $i=>$val)
		{
			if (!is_string($i)) $i=strval($i);

			if ($val!='')
			{
				$GLOBALS['SITE_DB']->query_insert('redirects',array(
					'r_from_page'=>post_param('from_page_'.$i),
					'r_from_zone'=>post_param('from_zone_'.$i),
					'r_to_page'=>post_param('to_page_'.$i),
					'r_to_zone'=>post_param('to_zone_'.$i),
					'r_is_transparent'=>post_param_integer('is_transparent_'.$i,0)
				),false,true); // Avoid problem when same key entered twice
			}
		}

		require_code('view_modes');
		erase_tempcode_cache();

		// Personal notes
		if (!is_null(post_param('notes',NULL)))
		{
			$notes=post_param('notes');
			set_long_value('notes',$notes);
		}

		// Redirect them back to editing screen
		$url=build_url(array('page'=>'_SELF','type'=>'misc'),'_SELF');
		return redirect_screen($title,$url,do_lang_tempcode('SUCCESS'));
	}

}


